@@ -2,7 +2,7 @@
 ** Copyright (C) 1989, 1991 by Jef Poskanzer.
 ** Copyright (C) 1997, 2000, 2002 by Greg Roelofs; based on an idea by
 **                                Stefan Schneider.
-** © 2009-2013 by Kornel Lesinski.
+** © 2009-2015 by Kornel Lesinski.
 **
 ** Permission to use, copy, modify, and distribute this software and its
 ** documentation for any purpose and without fee is hereby granted, provided
@@ -21,7 +21,7 @@
 
 #define index_of_channel(ch) (offsetof(f_pixel,ch)/sizeof(float))
 
-static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], float min_opaque_val, const f_pixel center);
+static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], const f_pixel center);
 
 struct box {
     f_pixel color;
@@ -31,11 +31,11 @@ struct box {
     unsigned int colors;
 };
 
-inline static double variance_diff(double val, const double good_enough) ALWAYS_INLINE;
+ALWAYS_INLINE static double variance_diff(double val, const double good_enough);
 inline static double variance_diff(double val, const double good_enough)
 {
     val *= val;
-    if (val < good_enough*good_enough) return val*0.5;
+    if (val < good_enough*good_enough) return val*0.25;
     return val;
 }
 
@@ -76,7 +76,7 @@ static double box_max_error(const hist_item achv[], const struct box *box)
     return max_error;
 }
 
-inline static double color_weight(f_pixel median, hist_item h) ALWAYS_INLINE;
+ALWAYS_INLINE static double color_weight(f_pixel median, hist_item h);
 
 static inline void hist_item_swap(hist_item *l, hist_item *r)
 {
@@ -87,18 +87,20 @@ static inline void hist_item_swap(hist_item *l, hist_item *r)
     }
 }
 
-inline static unsigned int qsort_pivot(const hist_item *const base, const unsigned int len) ALWAYS_INLINE;
+ALWAYS_INLINE static unsigned int qsort_pivot(const hist_item *const base, const unsigned int len);
 inline static unsigned int qsort_pivot(const hist_item *const base, const unsigned int len)
 {
-    if (len < 32) return len/2;
+    if (len < 32) {
+        return len/2;
+    }
 
     const unsigned int aidx=8, bidx=len/2, cidx=len-1;
-    const unsigned int a=base[aidx].sort_value, b=base[bidx].sort_value, c=base[cidx].sort_value;
+    const unsigned int a=base[aidx].tmp.sort_value, b=base[bidx].tmp.sort_value, c=base[cidx].tmp.sort_value;
     return (a < b) ? ((b < c) ? bidx : ((a < c) ? cidx : aidx ))
                    : ((b > c) ? bidx : ((a < c) ? aidx : cidx ));
 }
 
-inline static unsigned int qsort_partition(hist_item *const base, const unsigned int len) ALWAYS_INLINE;
+ALWAYS_INLINE static unsigned int qsort_partition(hist_item *const base, const unsigned int len);
 inline static unsigned int qsort_partition(hist_item *const base, const unsigned int len)
 {
     unsigned int l = 1, r = len;
@@ -106,12 +108,12 @@ inline static unsigned int qsort_partition(hist_item *const base, const unsigned
         hist_item_swap(&base[0], &base[qsort_pivot(base,len)]);
     }
 
-    const unsigned int pivot_value = base[0].sort_value;
+    const unsigned int pivot_value = base[0].tmp.sort_value;
     while (l < r) {
-        if (base[l].sort_value >= pivot_value) {
+        if (base[l].tmp.sort_value >= pivot_value) {
             l++;
         } else {
-            while(l < --r && base[r].sort_value <= pivot_value) {}
+            while(l < --r && base[r].tmp.sort_value <= pivot_value) {}
             hist_item_swap(&base[l], &base[r]);
         }
     }
@@ -201,7 +203,7 @@ static double prepare_sort(struct box *b, hist_item achv[])
         const float *chans = (const float *)&achv[b->ind + i].acolor;
         // Only the first channel really matters. When trying median cut many times
         // with different histogram weights, I don't want sort randomness to influence outcome.
-        achv[b->ind + i].sort_value = ((unsigned int)(chans[channels[0].chan]*65535.0)<<16) |
+        achv[b->ind + i].tmp.sort_value = ((unsigned int)(chans[channels[0].chan]*65535.0)<<16) |
                                        (unsigned int)((chans[channels[2].chan] + chans[channels[1].chan]/2.0 + chans[channels[3].chan]/4.0)*65535.0);
     }
 
@@ -226,7 +228,7 @@ static f_pixel get_median(const struct box *b, hist_item achv[])
 
     // technically the second color is not guaranteed to be sorted correctly
     // but most of the time it is good enough to be useful
-    return averagepixels(2, &achv[b->ind + median_start], 1.0, (f_pixel){0.5,0.5,0.5,0.5});
+    return averagepixels(2, &achv[b->ind + median_start], (f_pixel){0.5,0.5,0.5,0.5});
 }
 
 /*
@@ -236,7 +238,9 @@ static int best_splittable_box(struct box* bv, unsigned int boxes, const double
 {
     int bi=-1; double maxsum=0;
     for(unsigned int i=0; i < boxes; i++) {
-        if (bv[i].colors < 2) continue;
+        if (bv[i].colors < 2) {
+            continue;
+        }
 
         // looks only at max variance, because it's only going to split by it
         const double cv = MAX(bv[i].variance.r, MAX(bv[i].variance.g,bv[i].variance.b));
@@ -262,7 +266,7 @@ inline static double color_weight(f_pixel median, hist_item h)
     return sqrt(diff) * (sqrt(1.0+h.adjusted_weight)-1.0);
 }
 
-static colormap *colormap_from_boxes(struct box* bv, unsigned int boxes, hist_item *achv);
+static void set_colormap_from_boxes(colormap *map, struct box* bv, unsigned int boxes, hist_item *achv);
 static void adjust_histogram(hist_item *achv, const colormap *map, const struct box* bv, unsigned int boxes);
 
 double box_error(const struct box *box, const hist_item achv[])
@@ -307,7 +311,7 @@ static bool total_box_error_below_target(double target_mse, struct box bv[], uns
  ** on Paul Heckbert's paper, "Color Image Quantization for Frame Buffer
  ** Display," SIGGRAPH 1982 Proceedings, page 297.
  */
-LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, unsigned int newcolors, const double target_mse, const double max_mse)
+LIQ_PRIVATE colormap *mediancut(histogram *hist, unsigned int newcolors, const double target_mse, const double max_mse, void* (*malloc)(size_t), void (*free)(void*))
 {
     hist_item *achv = hist->achv;
     struct box bv[newcolors];
@@ -317,7 +321,7 @@ LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, uns
      */
     bv[0].ind = 0;
     bv[0].colors = hist->size;
-    bv[0].color = averagepixels(bv[0].colors, &achv[bv[0].ind], min_opaque_val, (f_pixel){0.5,0.5,0.5,0.5});
+    bv[0].color = averagepixels(bv[0].colors, &achv[bv[0].ind], (f_pixel){0.5,0.5,0.5,0.5});
     bv[0].variance = box_variance(achv, &bv[0]);
     bv[0].max_error = box_max_error(achv, &bv[0]);
     bv[0].sum = 0;
@@ -326,19 +330,11 @@ LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, uns
 
     unsigned int boxes = 1;
 
-    // remember smaller palette for fast searching
-    colormap *representative_subset = NULL;
-    unsigned int subset_size = ceilf(powf(newcolors,0.7f));
-
     /*
      ** Main loop: split boxes until we have enough.
      */
     while (boxes < newcolors) {
 
-        if (boxes == subset_size) {
-            representative_subset = colormap_from_boxes(bv, boxes, achv);
-        }
-
         // first splits boxes that exceed quality limit (to have colors for things like odd green pixel),
         // later raises the limit to allow large smooth areas/gradients get colors.
         const double current_max_mse = max_mse + (boxes/(double)newcolors)*16.0*max_mse;
@@ -378,14 +374,14 @@ LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, uns
         const f_pixel previous_center = bv[bi].color;
         bv[bi].colors = break_at;
         bv[bi].sum = lowersum;
-        bv[bi].color = averagepixels(bv[bi].colors, &achv[bv[bi].ind], min_opaque_val, previous_center);
+        bv[bi].color = averagepixels(bv[bi].colors, &achv[bv[bi].ind], previous_center);
         bv[bi].total_error = -1;
         bv[bi].variance = box_variance(achv, &bv[bi]);
         bv[bi].max_error = box_max_error(achv, &bv[bi]);
         bv[boxes].ind = indx + break_at;
         bv[boxes].colors = clrs - break_at;
         bv[boxes].sum = sm - lowersum;
-        bv[boxes].color = averagepixels(bv[boxes].colors, &achv[bv[boxes].ind], min_opaque_val, previous_center);
+        bv[boxes].color = averagepixels(bv[boxes].colors, &achv[bv[boxes].ind], previous_center);
         bv[boxes].total_error = -1;
         bv[boxes].variance = box_variance(achv, &bv[boxes]);
         bv[boxes].max_error = box_max_error(achv, &bv[boxes]);
@@ -397,14 +393,15 @@ LIQ_PRIVATE colormap *mediancut(histogram *hist, const float min_opaque_val, uns
         }
     }
 
-    colormap *map = colormap_from_boxes(bv, boxes, achv);
-    map->subset_palette = representative_subset;
+    colormap *map = pam_colormap(boxes, malloc, free);
+    set_colormap_from_boxes(map, bv, boxes, achv);
+
     adjust_histogram(achv, map, bv, boxes);
 
     return map;
 }
 
-static colormap *colormap_from_boxes(struct box* bv, unsigned int boxes, hist_item *achv)
+static void set_colormap_from_boxes(colormap *map, struct box* bv, unsigned int boxes, hist_item *achv)
 {
     /*
      ** Ok, we've got enough boxes.  Now choose a representative color for
@@ -414,8 +411,6 @@ static colormap *colormap_from_boxes(struct box* bv, unsigned int boxes, hist_it
      ** the box - this is the method specified in Heckbert's paper.
      */
 
-    colormap *map = pam_colormap(boxes);
-
     for(unsigned int bi = 0; bi < boxes; ++bi) {
         map->palette[bi].acolor = bv[bi].color;
 
@@ -425,8 +420,6 @@ static colormap *colormap_from_boxes(struct box* bv, unsigned int boxes, hist_it
             map->palette[bi].popularity += achv[i].perceptual_weight;
         }
     }
-
-    return map;
 }
 
 /* increase histogram popularity by difference from the final color (this is used as part of feedback loop) */
@@ -435,12 +428,12 @@ static void adjust_histogram(hist_item *achv, const colormap *map, const struct
     for(unsigned int bi = 0; bi < boxes; ++bi) {
         for(unsigned int i=bv[bi].ind; i < bv[bi].ind+bv[bi].colors; i++) {
             achv[i].adjusted_weight *= sqrt(1.0 +colordifference(map->palette[bi].acolor, achv[i].acolor)/4.0);
-            achv[i].likely_colormap_index = bi;
+            achv[i].tmp.likely_colormap_index = bi;
         }
     }
 }
 
-static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], const float min_opaque_val, const f_pixel center)
+static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], const f_pixel center)
 {
     double r = 0, g = 0, b = 0, a = 0, new_a=0, sum = 0;
     float maxa = 0;
@@ -457,9 +450,6 @@ static f_pixel averagepixels(unsigned int clrs, const hist_item achv[], const fl
 
     if (sum) new_a /= sum;
 
-    /** if there was at least one completely opaque color, "round" final color to opaque */
-    if (new_a >= min_opaque_val && maxa >= (255.0/256.0)) new_a = 1;
-
     sum=0;
     // reverse iteration for cache locality with previous loop
     for(int i = clrs-1; i >= 0; i--) {
