@@ -6,7 +6,7 @@
 #define VITER_CACHE_LINE_GAP ((64+sizeof(viter_state)-1)/sizeof(viter_state))
 
 typedef struct {
-    float a, r, g, b, total;
+    double a, r, g, b, total;
 } viter_state;
 
 typedef void (*viter_callback)(hist_item *item, float diff);
@@ -14,6 +14,6 @@ typedef void (*viter_callback)(hist_item *item, float diff);
 LIQ_PRIVATE void viter_init(const colormap *map, const unsigned int max_threads, viter_state state[]);
 LIQ_PRIVATE void viter_update_color(const f_pixel acolor, const float value, const colormap *map, unsigned int match, const unsigned int thread, viter_state average_color[]);
 LIQ_PRIVATE void viter_finalize(colormap *map, const unsigned int max_threads, const viter_state state[]);
-LIQ_PRIVATE double viter_do_iteration(histogram *hist, colormap *const map, const float min_opaque_val, viter_callback callback, const bool fast_palette);
+LIQ_PRIVATE double viter_do_iteration(histogram *hist, colormap *const map, viter_callback callback, const bool fast_palette);
 
 #endif
