@@ -1,9 +1,6 @@
 /* pngquant.c - quantize the colors in an alphamap down to a specified number
 **
 ** Copyright (C) 1989, 1991 by Jef Poskanzer.
-** Copyright (C) 1997, 2000, 2002 by Greg Roelofs; based on an idea by
-**                                Stefan Schneider.
-** © 2009-2013 by Kornel Lesinski.
 **
 ** Permission to use, copy, modify, and distribute this software and its
 ** documentation for any purpose and without fee is hereby granted, provided
@@ -11,18 +8,51 @@
 ** copyright notice and this permission notice appear in supporting
 ** documentation.  This software is provided "as is" without express or
 ** implied warranty.
+**
+** - - - -
+**
+** © 1997-2002 by Greg Roelofs; based on an idea by Stefan Schneider.
+** © 2009-2015 by Kornel Lesiński.
+**
+** All rights reserved.
+**
+** Redistribution and use in source and binary forms, with or without modification,
+** are permitted provided that the following conditions are met:
+**
+** 1. Redistributions of source code must retain the above copyright notice,
+**    this list of conditions and the following disclaimer.
+**
+** 2. Redistributions in binary form must reproduce the above copyright notice,
+**    this list of conditions and the following disclaimer in the documentation
+**    and/or other materials provided with the distribution.
+**
+** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+** SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+** CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+** OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+**
 */
 
-#define PNGQUANT_VERSION "2.0.1 (September 2013)"
+#define PNGQUANT_VERSION LIQ_VERSION_STRING " (June 2015)"
 
 #define PNGQUANT_USAGE "\
-usage:  pngquant [options] [ncolors] [pngfile [pngfile ...]]\n\n\
+usage:  pngquant [options] [ncolors] -- pngfile [pngfile ...]\n\
+        pngquant [options] [ncolors] - >stdout <stdin\n\n\
 options:\n\
   --force           overwrite existing output files (synonym: -f)\n\
-  --nofs            disable Floyd-Steinberg dithering\n\
-  --ext new.png     set custom suffix/extension for output filename\n\
+  --skip-if-larger  only save converted files if they're smaller than original\n\
+  --output file     output path, only if one input file is specified (synonym: -o)\n\
+  --ext new.png     set custom suffix/extension for output filenames\n\
+  --quality min-max don't save below min, use fewer colors below max (0-100)\n\
   --speed N         speed/quality trade-off. 1=slow, 3=default, 11=fast & rough\n\
-  --quality min-max don't save below min, use less colors below max (0-100)\n\
+  --nofs            disable Floyd-Steinberg dithering\n\
+  --posterize N     output lower resolution color (e.g. for ARGB4444 output)\n\
   --verbose         print status messages (synonym: -v)\n\
 \n\
 Quantizes one or more 32-bit RGBA PNGs to 8-bit (or smaller) RGBA-palette\n\
@@ -31,7 +61,7 @@ The output filename is the same as the input name except that\n\
 it ends in \"-fs8.png\", \"-or8.png\" or your custom extension (unless the\n\
 input is stdin, in which case the quantized image will go to stdout).\n\
 The default behavior if the output file exists is to skip the conversion;\n\
-use --force to overwrite.\n"
+use --force to overwrite. See man page for full list of options.\n"
 
 
 #include <stdio.h>
@@ -40,6 +70,10 @@ use --force to overwrite.\n"
 #include <stdarg.h>
 #include <stdbool.h>
 #include <getopt.h>
+#include <unistd.h>
+
+extern char *optarg;
+extern int optind, opterr;
 
 #if defined(WIN32) || defined(__WIN32__)
 #  include <fcntl.h>    /* O_BINARY */
@@ -55,7 +89,6 @@ use --force to overwrite.\n"
 
 #include "rwpng.h"  /* typedefs, common macros, public prototypes */
 #include "lib/libimagequant.h"
-#include "lib/pam.h"
 
 struct pngquant_options {
     liq_attr *liq;
@@ -63,13 +96,14 @@ struct pngquant_options {
     liq_log_callback_function *log_callback;
     void *log_callback_user_info;
     float floyd;
-    bool using_stdin, force, ie_mode, min_quality_limit, fast_compression;
+    bool using_stdin, using_stdout, force, fast_compression, ie_mode,
+        min_quality_limit, skip_if_larger,
+        verbose;
 };
 
 static pngquant_error prepare_output_image(liq_result *result, liq_image *input_image, png8_image *output_image);
 static void set_palette(liq_result *result, png8_image *output_image);
-static void pngquant_output_image_free(png8_image *output_image);
-static pngquant_error read_image(liq_attr *options, const char *filename, int using_stdin, png24_image *input_image_p, liq_image **liq_image_p, bool keep_input_pixels);
+static pngquant_error read_image(liq_attr *options, const char *filename, int using_stdin, png24_image *input_image_p, liq_image **liq_image_p, bool keep_input_pixels, bool verbose);
 static pngquant_error write_image(png8_image *output_image, png24_image *output_image24, const char *outname, struct pngquant_options *options);
 static char *add_filename_extension(const char *filename, const char *newext);
 static bool file_exists(const char *outname);
@@ -108,6 +142,7 @@ static void log_callback_buferred_flush(const liq_attr *attr, void *context)
     struct buffered_log *log = context;
     if (log->buf_used) {
         fwrite(log->buf, 1, log->buf_used, stderr);
+        fflush(stderr);
         log->buf_used = 0;
     }
 }
@@ -130,10 +165,10 @@ static void print_full_version(FILE *fd)
 {
     fprintf(fd, "pngquant, %s, by Greg Roelofs, Kornel Lesinski.\n"
         #ifndef NDEBUG
-                    "   DEBUG (slow) version.\n"
+                    "   DEBUG (slow) version.\n" /* NDEBUG disables assert() */
         #endif
         #if USE_SSE
-                    "   Compiled with SSE2 instructions.\n"
+                    "   Compiled with SSE instructions.\n"
         #endif
         #if _OPENMP
                     "   Compiled with OpenMP (multicore support).\n"
@@ -219,7 +254,8 @@ static void fix_obsolete_options(const unsigned int argc, char *argv[])
     }
 }
 
-enum {arg_floyd=1, arg_ordered, arg_ext, arg_no_force, arg_iebug, arg_transbug, arg_map, arg_posterize};
+enum {arg_floyd=1, arg_ordered, arg_ext, arg_no_force, arg_iebug,
+    arg_transbug, arg_map, arg_posterize, arg_skip_larger};
 
 static const struct option long_options[] = {
     {"verbose", no_argument, NULL, 'v'},
@@ -232,6 +268,7 @@ static const struct option long_options[] = {
     {"iebug", no_argument, NULL, arg_iebug},
     {"transbug", no_argument, NULL, arg_transbug},
     {"ext", required_argument, NULL, arg_ext},
+    {"skip-if-larger", no_argument, NULL, arg_skip_larger},
     {"output", required_argument, NULL, 'o'},
     {"speed", required_argument, NULL, 's'},
     {"quality", required_argument, NULL, 'Q'},
@@ -252,13 +289,10 @@ int main(int argc, char *argv[])
     };
     options.liq = liq_attr_create();
 
-#if USE_SSE
     if (!options.liq) {
-        print_full_version(stderr);
-        fputs("SSE2-capable CPU is required for this build.\n", stderr);
+        fputs("SSE-capable CPU is required for this build.\n", stderr);
         return WRONG_ARCHITECTURE;
     }
-#endif
 
     unsigned int error_count=0, skipped_count=0, file_count=0;
     pngquant_error latest_error=SUCCESS;
@@ -271,22 +305,21 @@ int main(int argc, char *argv[])
         opt = getopt_long(argc, argv, "Vvqfhs:Q:o:", long_options, NULL);
         switch (opt) {
             case 'v':
-                liq_set_log_callback(options.liq, log_callback, NULL);
-                options.log_callback = log_callback;
+                options.verbose = true;
                 break;
             case 'q':
-                liq_set_log_callback(options.liq, NULL, NULL);
-                options.log_callback = NULL;
+                options.verbose = false;
                 break;
 
             case arg_floyd:
                 options.floyd = optarg ? atof(optarg) : 1.0;
-                if (options.floyd < 0 || options.floyd > 1.0) {
+                if (options.floyd < 0 || options.floyd > 1.f) {
                     fputs("--floyd argument must be in 0..1 range\n", stderr);
                     return INVALID_ARGUMENT;
                 }
                 break;
             case arg_ordered: options.floyd = 0; break;
+
             case 'f': options.force = true; break;
             case arg_no_force: options.force = false; break;
 
@@ -301,13 +334,17 @@ int main(int argc, char *argv[])
             case arg_iebug:
                 // opacities above 238 will be rounded up to 255, because IE6 truncates <255 to 0.
                 liq_set_min_opacity(options.liq, 238);
-                options.ie_mode = true;
+                fputs("  warning: the workaround for IE6 is deprecated\n", stderr);
                 break;
 
             case arg_transbug:
                 liq_set_last_index_transparent(options.liq, true);
                 break;
 
+            case arg_skip_larger:
+                options.skip_if_larger = true;
+                break;
+
             case 's':
                 {
                     int speed = atoi(optarg);
@@ -342,8 +379,8 @@ int main(int argc, char *argv[])
             case arg_map:
                 {
                     png24_image tmp = {};
-                    if (SUCCESS != read_image(options.liq, optarg, false, &tmp, &options.fixed_palette_image, false)) {
-                        fprintf(stderr, "  error: Unable to load %s", optarg);
+                    if (SUCCESS != read_image(options.liq, optarg, false, &tmp, &options.fixed_palette_image, false, false)) {
+                        fprintf(stderr, "  error: unable to load %s", optarg);
                         return INVALID_ARGUMENT;
                     }
                 }
@@ -369,14 +406,19 @@ int main(int argc, char *argv[])
 
     if (argn >= argc) {
         if (argn > 1) {
-            fputs("No input files specified. See -h for help.\n", stderr);
+            fputs("No input files specified.\n", stderr);
         } else {
             print_full_version(stderr);
-            print_usage(stderr);
         }
+        print_usage(stderr);
         return MISSING_ARGUMENT;
     }
 
+    if (options.verbose) {
+        liq_set_log_callback(options.liq, log_callback, NULL);
+        options.log_callback = log_callback;
+    }
+
     char *colors_end;
     unsigned long colors = strtoul(argv[argn], &colors_end, 10);
     if (colors_end != argv[argn] && '\0' == colors_end[0]) {
@@ -395,19 +437,17 @@ int main(int argc, char *argv[])
     // new filename extension depends on options used. Typically basename-fs8.png
     if (newext == NULL) {
         newext = options.floyd > 0 ? "-ie-fs8.png" : "-ie-or8.png";
-        if (!options.ie_mode) newext += 3; /* skip "-ie" */
+        if (!options.ie_mode) {
+            newext += 3;    /* skip "-ie" */
+        }
     }
 
     if (argn == argc || (argn == argc-1 && 0==strcmp(argv[argn],"-"))) {
         options.using_stdin = true;
+        options.using_stdout = !output_file_path;
         argn = argc-1;
     }
 
-    if (options.using_stdin && output_file_path) {
-        fputs("--output can't be mixed with stdin\n", stderr);
-        return INVALID_ARGUMENT;
-    }
-
     const int num_files = argc-argn;
 
     if (output_file_path && num_files != 1) {
@@ -426,7 +466,7 @@ int main(int argc, char *argv[])
 #endif
 
     #pragma omp parallel for \
-        schedule(dynamic) reduction(+:skipped_count) reduction(+:error_count) reduction(+:file_count) shared(latest_error)
+        schedule(static, 1) reduction(+:skipped_count) reduction(+:error_count) reduction(+:file_count) shared(latest_error)
     for(int i=0; i < num_files; i++) {
         struct pngquant_options opts = options;
         opts.liq = liq_attr_copy(options.liq);
@@ -448,17 +488,17 @@ int main(int argc, char *argv[])
 
         const char *outname = output_file_path;
         char *outname_free = NULL;
-        if (!options.using_stdin) {
+        if (!options.using_stdout) {
             if (!outname) {
                 outname = outname_free = add_filename_extension(filename, newext);
             }
             if (!options.force && file_exists(outname)) {
-                fprintf(stderr, "  error:  %s exists; not overwriting\n", outname);
+                fprintf(stderr, "  error: '%s' exists; not overwriting\n", outname);
                 retval = NOT_OVERWRITING_ERROR;
             }
         }
 
-        if (!retval) {
+        if (SUCCESS == retval) {
             retval = pngquant_file(filename, outname, &opts);
         }
 
@@ -471,7 +511,7 @@ int main(int argc, char *argv[])
             {
                 latest_error = retval;
             }
-            if (retval == TOO_LOW_QUALITY) {
+            if (retval == TOO_LOW_QUALITY || retval == TOO_LARGE_FILE) {
                 skipped_count++;
             } else {
                 error_count++;
@@ -499,16 +539,6 @@ int main(int argc, char *argv[])
     return latest_error;
 }
 
-
-static void pngquant_output_image_free(png8_image *output_image)
-{
-    free(output_image->indexed_data);
-    output_image->indexed_data = NULL;
-
-    free(output_image->row_pointers);
-    output_image->row_pointers = NULL;
-}
-
 pngquant_error pngquant_file(const char *filename, const char *outname, struct pngquant_options *options)
 {
     pngquant_error retval = SUCCESS;
@@ -517,15 +547,30 @@ pngquant_error pngquant_file(const char *filename, const char *outname, struct p
 
     liq_image *input_image = NULL;
     png24_image input_image_rwpng = {};
-    bool keep_input_pixels = options->using_stdin && options->min_quality_limit; // original may need to be output to stdout
-    if (!retval) {
-        retval = read_image(options->liq, filename, options->using_stdin, &input_image_rwpng, &input_image, keep_input_pixels);
+    bool keep_input_pixels = options->skip_if_larger || (options->using_stdout && options->min_quality_limit); // original may need to be output to stdout
+    if (SUCCESS == retval) {
+        retval = read_image(options->liq, filename, options->using_stdin, &input_image_rwpng, &input_image, keep_input_pixels, options->verbose);
     }
 
+    int quality_percent = 90; // quality on 0-100 scale, updated upon successful remap
     png8_image output_image = {};
-    if (!retval) {
-        verbose_printf(options, "  read %luKB file corrected for gamma %2.1f",
-                       (input_image_rwpng.file_size+1023UL)/1024UL, 1.0/input_image_rwpng.gamma);
+    if (SUCCESS == retval) {
+        verbose_printf(options, "  read %luKB file", (input_image_rwpng.file_size+1023UL)/1024UL);
+
+#if USE_LCMS
+        if (input_image_rwpng.lcms_status == ICCP) {
+            verbose_printf(options, "  used embedded ICC profile to transform image to sRGB colorspace");
+        } else if (input_image_rwpng.lcms_status == GAMA_CHRM) {
+            verbose_printf(options, "  used gAMA and cHRM chunks to transform image to sRGB colorspace");
+        } else if (input_image_rwpng.lcms_status == ICCP_WARN_GRAY) {
+            verbose_printf(options, "  warning: ignored ICC profile in GRAY colorspace");
+        }
+#endif
+
+        if (input_image_rwpng.gamma != 0.45455) {
+            verbose_printf(options, "  corrected image from gamma %2.1f to sRGB gamma",
+                           1.0/input_image_rwpng.gamma);
+        }
 
         // when using image as source of a fixed palette the palette is extracted using regular quantization
         liq_result *remap = liq_quantize_image(options->liq, options->fixed_palette_image ? options->fixed_palette_image : input_image);
@@ -535,7 +580,7 @@ pngquant_error pngquant_file(const char *filename, const char *outname, struct p
             liq_set_dithering_level(remap, options->floyd);
 
             retval = prepare_output_image(remap, input_image, &output_image);
-            if (!retval) {
+            if (SUCCESS == retval) {
                 if (LIQ_OK != liq_write_remapped_image_rows(remap, input_image, output_image.row_pointers)) {
                     retval = OUT_OF_MEMORY_ERROR;
                 }
@@ -544,7 +589,8 @@ pngquant_error pngquant_file(const char *filename, const char *outname, struct p
 
                 double palette_error = liq_get_quantization_error(remap);
                 if (palette_error >= 0) {
-                    verbose_printf(options, "  mapped image to new colors...MSE=%.3f (Q=%d)", palette_error, liq_get_quantization_quality(remap));
+                    quality_percent = liq_get_quantization_quality(remap);
+                    verbose_printf(options, "  mapped image to new colors...MSE=%.3f (Q=%d)", palette_error, quality_percent);
                 }
             }
             liq_result_destroy(remap);
@@ -553,21 +599,36 @@ pngquant_error pngquant_file(const char *filename, const char *outname, struct p
         }
     }
 
-    if (!retval) {
+    if (SUCCESS == retval) {
+
+        if (options->skip_if_larger) {
+            // this is very rough approximation, but generally avoid losing more quality than is gained in file size.
+            // Quality is squared, because even greater savings are needed to justify big quality loss.
+            double quality = quality_percent/100.0;
+            output_image.maximum_file_size = (input_image_rwpng.file_size-1) * quality*quality;
+        }
+
         output_image.fast_compression = options->fast_compression;
+        output_image.chunks = input_image_rwpng.chunks; input_image_rwpng.chunks = NULL;
         retval = write_image(&output_image, NULL, outname, options);
-    } else if (TOO_LOW_QUALITY == retval && options->using_stdin) {
+
+        if (TOO_LARGE_FILE == retval) {
+            verbose_printf(options, "  file exceeded expected size of %luKB", (unsigned long)output_image.maximum_file_size/1024UL);
+        }
+    }
+
+    if (options->using_stdout && keep_input_pixels && (TOO_LARGE_FILE == retval || TOO_LOW_QUALITY == retval)) {
         // when outputting to stdout it'd be nasty to create 0-byte file
         // so if quality is too low, output 24-bit original
         pngquant_error write_retval = write_image(NULL, &input_image_rwpng, outname, options);
-        if (write_retval) retval = write_retval;
+        if (write_retval) {
+            retval = write_retval;
+        }
     }
 
-    liq_image_destroy(input_image);
-    pngquant_output_image_free(&output_image);
-
-    free(input_image_rwpng.row_pointers);
-    free(input_image_rwpng.rgba_data);
+    if (input_image) liq_image_destroy(input_image);
+    rwpng_free_image24(&input_image_rwpng);
+    rwpng_free_image8(&output_image);
 
     return retval;
 }
@@ -611,10 +672,23 @@ static char *add_filename_extension(const char *filename, const char *newext)
     if (!outname) return NULL;
 
     strncpy(outname, filename, x);
-    if (strncmp(outname+x-4, ".png", 4) == 0 || strncmp(outname+x-4, ".PNG", 4) == 0)
+    if (strncmp(outname+x-4, ".png", 4) == 0 || strncmp(outname+x-4, ".PNG", 4) == 0) {
         strcpy(outname+x-4, newext);
-    else
+    } else {
         strcpy(outname+x, newext);
+    }
+
+    return outname;
+}
+
+static char *temp_filename(const char *basename) {
+    size_t x = strlen(basename);
+
+    char *outname = malloc(x+1+4);
+    if (!outname) return NULL;
+
+    strcpy(outname, basename);
+    strcpy(outname+x, ".tmp");
 
     return outname;
 }
@@ -626,10 +700,32 @@ static void set_binary_mode(FILE *fp)
 #endif
 }
 
+static const char *filename_part(const char *path)
+{
+    const char *outfilename = strrchr(path, '/');
+    if (outfilename) {
+        return outfilename+1;
+    } else {
+        return path;
+    }
+}
+
+static bool replace_file(const char *from, const char *to, const bool force) {
+#if defined(WIN32) || defined(__WIN32__)
+    if (force) {
+        // On Windows rename doesn't replace
+        unlink(to);
+    }
+#endif
+    return (0 == rename(from, to));
+}
+
 static pngquant_error write_image(png8_image *output_image, png24_image *output_image24, const char *outname, struct pngquant_options *options)
 {
     FILE *outfile;
-    if (options->using_stdin) {
+    char *tempname = NULL;
+
+    if (options->using_stdout) {
         set_binary_mode(stdout);
         outfile = stdout;
 
@@ -639,19 +735,19 @@ static pngquant_error write_image(png8_image *output_image, png24_image *output_
             verbose_printf(options, "  writing truecolor image to stdout");
         }
     } else {
+        tempname = temp_filename(outname);
+        if (!tempname) return OUT_OF_MEMORY_ERROR;
 
-        if ((outfile = fopen(outname, "wb")) == NULL) {
-            fprintf(stderr, "  error:  cannot open %s for writing\n", outname);
+        if ((outfile = fopen(tempname, "wb")) == NULL) {
+            fprintf(stderr, "  error: cannot open '%s' for writing\n", tempname);
+            free(tempname);
             return CANT_WRITE_ERROR;
         }
 
-        const char *outfilename = strrchr(outname, '/');
-        if (outfilename) outfilename++; else outfilename = outname;
-
         if (output_image) {
-            verbose_printf(options, "  writing %d-color image as %s", output_image->num_palette, outfilename);
+            verbose_printf(options, "  writing %d-color image as %s", output_image->num_palette, filename_part(outname));
         } else {
-            verbose_printf(options, "  writing truecolor image as %s", outfilename);
+            verbose_printf(options, "  writing truecolor image as %s", filename_part(outname));
         }
     }
 
@@ -665,17 +761,31 @@ static pngquant_error write_image(png8_image *output_image, png24_image *output_
         }
     }
 
-    if (retval) {
+    if (!options->using_stdout) {
+        fclose(outfile);
+
+        if (SUCCESS == retval) {
+            // Image has been written to a temporary file and then moved over destination.
+            // This makes replacement atomic and avoids damaging destination file on write error.
+            if (!replace_file(tempname, outname, options->force)) {
+                retval = CANT_WRITE_ERROR;
+            }
+        }
+
+        if (retval) {
+            unlink(tempname);
+        }
+    }
+    free(tempname);
+
+    if (retval && retval != TOO_LARGE_FILE) {
         fprintf(stderr, "  error: failed writing image to %s\n", outname);
     }
 
-    if (!options->using_stdin)
-        fclose(outfile);
-
     return retval;
 }
 
-static pngquant_error read_image(liq_attr *options, const char *filename, int using_stdin, png24_image *input_image_p, liq_image **liq_image_p, bool keep_input_pixels)
+static pngquant_error read_image(liq_attr *options, const char *filename, int using_stdin, png24_image *input_image_p, liq_image **liq_image_p, bool keep_input_pixels, bool verbose)
 {
     FILE *infile;
 
@@ -690,14 +800,15 @@ static pngquant_error read_image(liq_attr *options, const char *filename, int us
     pngquant_error retval;
     #pragma omp critical (libpng)
     {
-        retval = rwpng_read_image24(infile, input_image_p);
+        retval = rwpng_read_image24(infile, input_image_p, verbose);
     }
 
-    if (!using_stdin)
+    if (!using_stdin) {
         fclose(infile);
+    }
 
     if (retval) {
-        fprintf(stderr, "  error: rwpng_read_image() error %d\n", retval);
+        fprintf(stderr, "  error: cannot decode image %s\n", using_stdin ? "from stdin" : filename_part(filename));
         return retval;
     }
 
@@ -708,7 +819,9 @@ static pngquant_error read_image(liq_attr *options, const char *filename, int us
     }
 
     if (!keep_input_pixels) {
-        liq_image_set_memory_ownership(*liq_image_p, LIQ_OWN_ROWS | LIQ_OWN_PIXELS);
+        if (LIQ_OK != liq_image_set_memory_ownership(*liq_image_p, LIQ_OWN_ROWS | LIQ_OWN_PIXELS)) {
+            return OUT_OF_MEMORY_ERROR;
+        }
         input_image_p->row_pointers = NULL;
         input_image_p->rgba_data = NULL;
     }
